<?php

namespace Teknisa\DF\Controllers;

use Estoque\Entities\Lanctoestoq;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Itemproimpos;
use Teknisa\DF\Services\LancamentoNotafiscalService;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;

class LancamentoNotafiscalController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $lancamentoNotafiscalService;
    protected $validacoesEstoqueService;
    protected $nfEntradaService;
    protected $parameterBagColumns = ["TODOS", "P_CDPRODFIN", "P_CDPRODINI", "CDFILIAL","P_CDFILIAL"];
    
    protected $dataSourceNames = array(
        'default'      => '',
        'filial'       => 'filial_view',
        'prod_fili'   => 'prodporfili_produto'
    );

    public function __construct($crudService, $parameterBag, LancamentoNotafiscalService $lancamentoNotafiscalService, ValidacoesEstoqueService $validacoesEstoqueService, $environment,  $entityManager)
    {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->lancamentoNotafiscalService = $lancamentoNotafiscalService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function montaCorpoNota (Request\Filter $request, Response $response) {
    	$requestConditions = $request->getFilterCriteria()->getConditions();
        $filial        = $requestConditions[0]["value"];
        $lancamento    = $requestConditions[1]["value"];
        $entrada_saida = $requestConditions[2]["value"];
        $statusNota    = $requestConditions[3]["value"];

        $arrayProdutos         = $this->lancamentoNotafiscalService->buscaProdutos($filial, $lancamento);
        $arrayImpostosProdutos = $this->lancamentoNotafiscalService->buscaImpostosProdutos($filial, $lancamento, $entrada_saida, $statusNota);
        $arrayImpostos         = $this->lancamentoNotafiscalService->buscaImpostos($filial, $lancamento, $entrada_saida, $statusNota);
        $arrayEncargos         = $this->lancamentoNotafiscalService->buscaEncargos($filial, $lancamento);
        $arrayVencimentos      = $this->lancamentoNotafiscalService->buscaVencimentos($filial, $lancamento, $entrada_saida);
        $arrayDadosAdicionais  = $this->lancamentoNotafiscalService->buscaDadosAdicionais($filial, $lancamento);
        $arrayItensRateionf    = $this->lancamentoNotafiscalService->buscaItensRateionf($filial, $lancamento);
        $arrayRateionf         = $this->lancamentoNotafiscalService->buscaRateionf($filial, $lancamento, $arrayItensRateionf);
        $arrayItensCombs       = $this->buscaItensCombs($filial, $lancamento);
        
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens'         , $arrayProdutos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto_item'  , $arrayImpostosProdutos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto'       , $arrayImpostos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_encargo'       , $arrayEncargos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_vencimento'    , $arrayVencimentos));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_dadosadic'     , $arrayDadosAdicionais));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens_rateionf', $arrayItensRateionf));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_rateionf'      , $arrayRateionf));
    	$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens_combs'   , $arrayItensCombs));
    }
    
    public function lancamento_nota_itens (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $arrayProdutos = $this->lancamentoNotafiscalService->buscaProdutos($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
    }
    
    public function lancamento_nota_dadosadic (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $arrayDadosAdicionais = $this->lancamentoNotafiscalService->buscaDadosAdicionais($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_dadosadic', $arrayDadosAdicionais));
    }
    
    public function lancamento_nota_encargo (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $arrayEncargos = $this->lancamentoNotafiscalService->buscaEncargos($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_encargo', $arrayEncargos));
    }
    
    public function lancamento_nota_imposto_item (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        $entrada_saida = $requestConditions[2]["value"];
        $statusNota    = $requestConditions[3]["value"];
        
        $arrayImpostosProdutos = $this->lancamentoNotafiscalService->buscaImpostosProdutos($filial, $lancamento, $entrada_saida, $statusNota);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto_item', $arrayImpostosProdutos));
    }
    
    public function lancamento_nota_imposto (Request\Filter $request, Response $response) {
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        $entrada_saida = $requestConditions[2]["value"];
        $statusNota    = $requestConditions[3]["value"];
        
        $arrayImpostos = $this->lancamentoNotafiscalService->buscaImpostos($filial, $lancamento, $entrada_saida, $statusNota);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_imposto', $arrayImpostos));
    }

    public function getItensRateio(Request\Filter $request, Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $filial = $requestConditions[0]["value"];
        $lancamento = $requestConditions[1]["value"];
        
        $itensRateio = $this->lancamentoNotafiscalService->buscaItensRateionf($filial, $lancamento);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('itens_rateio', $itensRateio));
    }

    public function excluirNotaSaida(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            
            $filial = $conditions[0]['value'];
            $lancamento = $conditions[1]['value'];

            $filter = array(
                array(
                    "columnName" => "IDDOCREPLIC",
                    "operator"=> "=",
                    "value"=> $filial.$lancamento
                )  
            );
            
            $notasReplicadas = $this->crudService->getFilteredDataSet('notaReplicada_tipooperac', $filter);
            $notasReplicadas = $notasReplicadas->getRows();
            
            if($notasReplicadas) {
                foreach ($notasReplicadas as $notaReplic) {
                    $this->lancamentoNotafiscalService->procedureDeletaNf($notaReplic['CDFILIAL'], $notaReplic['NRLANCTONF']);
                }
            }
            
            $this->lancamentoNotafiscalService->deletaRelacNotaOrigem($filial, $lancamento);
            
            $this->lancamentoNotafiscalService->validaExclusaoNotaUtilizaAf($filial, $lancamento);

            $lanctoestoq = Lanctoestoq::findOneByNrlanctonfAndCdfilial($lancamento, $filial);
            
            $this->lancamentoNotafiscalService->verificaDevolucaoOl($filial, $lancamento);

            $this->lancamentoNotafiscalService->procedureDeletaNf($filial, $lancamento);

            if (is_object($lanctoestoq)) {
                $lancamentoEstoque = $lanctoestoq->getNrlancestq();
                $this->validacoesEstoqueService->setLanctoestoq($lanctoestoq);
                $this->validacoesEstoqueService->atualizaEstoque($filial, $lancamentoEstoque);
            }
            
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('excluir_nota_saida', array('success' => 'S')));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarItem(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $item = $conditions[0]['value'];
            if(!isset($item['CDALMOXARIFE'])) $item['CDALMOXARIFE'] = ' ';
            else if($item['CDALMOXARIFE'] == null) $item['CDALMOXARIFE'] = ' ';
            
            if(!isset($item['CDLOCALESTOQ'])) $item['CDLOCALESTOQ'] = ' ';
            else if($item['CDLOCALESTOQ'] == null) $item['CDLOCALESTOQ'] = ' ';
            
            if(!isset($item['NRLOTEESTQ']))   $item['NRLOTEESTQ']   = ' ';
            else if($item['NRLOTEESTQ'] == null) $item['NRLOTEESTQ'] = ' ';
            
            if(!isset($item['NRSUBLOTE']))    $item['NRSUBLOTE']    = ' ';
            else if($item['NRSUBLOTE'] == null) $item['NRSUBLOTE'] = ' ';

            if(empty($item['CDSERVFATU'])) $item['CDSERVFATU'] =  null;
            if(empty($item['CDSERVFILI'])) $item['CDSERVFILI'] =  null;
            if(empty($item['DSINFADICPROD'])) $item['DSINFADICPROD'] = null;
            
            $this->lancamentoNotafiscalService->procedureGeraNotaFiscalProduto(
                $item['CDFILIAL'], $item['NRLANCTONF'], $item['CDPRODUTO'], $item['CDPRODEMIS'], $item['DSPRODEMIS'], $item['QTITEMPROD'], $item['VRUNITITEM'], $item['VRDESCITEM'], $item['CDALMOXARIFE'], $item['CDLOCALESTOQ'], 
                $item['NRLOTEESTQ'], $item['NRSUBLOTE'], $item['CDSERVFATU'], null, 'S', null, null, $item["CDSERVFILI"], $item['DSINFADICPROD']
            );
            
            $this->lancamentoNotafiscalService->salvaCFOPeCST($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRSEQITPROD'], $item['CDFISOPEITEM'], $item['CDSITUTRIBPR']);

//            $this->alteraStatusNotafiscal($item['CDFILIAL'], $item['NRLANCTONF'], $item['NRORG'], 'D');
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getLote(Request\Filter $request, Response $response){
        try{
            $conditions     = $request->getFilterCriteria()->getConditions();
            $P_IDUTILLOTE   = $conditions[0]['value'];
            $P_CDFILIAL     = $conditions[1]['value'];
            $P_CDALMOXARIFE = $conditions[2]['value'];
            $P_CDLOCALESTOQ = $conditions[3]['value'];
            $P_CDPRODUTO    = $conditions[4]['value'];
            
            if(!isset($P_CDALMOXARIFE )) $P_CDALMOXARIFE  = ' ';
            else if($P_CDALMOXARIFE  == null) $P_CDALMOXARIFE  = ' ';
            
            if(!isset($P_CDLOCALESTOQ)) $P_CDLOCALESTOQ = ' ';
            else if($P_CDLOCALESTOQ == null) $P_CDLOCALESTOQ = ' ';
            
            $loteSub = $this->lancamentoNotafiscalService->procedureGetLote(
                                                                                $P_IDUTILLOTE,  
                                                                                $P_CDFILIAL,
                                                                                $P_CDALMOXARIFE,
                                                                                $P_CDLOCALESTOQ,
                                                                                $P_CDPRODUTO 
                                                                            );
            $loteSub_value = array($loteSub);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getlote', $loteSub_value));
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function ConsultaPosEstDia(Request\Filter $request, Response $response){
        try{
            $conditions     = $request->getFilterCriteria()->getConditions();
            //var_dump($conditions);
            $P_CDFILIEMIT   = $conditions[0]['value'];
            $P_CDALMOXARIFE = $conditions[1]['value'];
            $P_CDLOCALESTOQ = $conditions[2]['value'];
            $P_NRLOTEESTQ   = $conditions[3]['value'];
            $P_NRSUBLOTE    = $conditions[4]['value'];
            $P_DTPOSICAO    = $conditions[5]['value']; // Data de saída da nf + 1
            $P_INTPCUSTO    = $conditions[6]['value'];
            $P_CDPRODUTO    = $conditions[7]['value'];
            $P_QTDE         = $conditions[8]['value'];
            
            $VrUnitario     = $this -> lancamentoNotafiscalService -> ConsultaPosEstDia(
                                                                                            $P_CDFILIEMIT,
                                                                                            $P_CDALMOXARIFE,
                                                                                            $P_CDLOCALESTOQ, 
                                                                                            $P_NRLOTEESTQ, 
                                                                                            $P_NRSUBLOTE,
                                                                                            $P_DTPOSICAO,// Data de saída da nf + 1
                                                                                            $P_INTPCUSTO, 
                                                                                            $P_CDPRODUTO,
                                                                                            $P_QTDE 
                                                                                        );
            $VrUnitario_value = array($VrUnitario);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('consulta_post_est_dia', $VrUnitario_value));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function alteraStatusNotafiscal($filial, $lancamento, $organizacao, $status){
        //@todo somente quando status nota !== 'D'
        $this->lancamentoNotafiscalService->updateStatusNotafiscal($filial, $lancamento, $organizacao, $status);
    }

    public function fecharNota(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $cabecalho = $conditions[0]['value'];
            $vencimentos = $conditions[1]['value'];
            $forcaFechamento = isset($conditions[2]) ? $conditions[2]['value'] : false;
            
            $infoNotaFiscal = $this->lancamentoNotafiscalService->fechaNotaFiscal($cabecalho, $vencimentos, $forcaFechamento);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('fecharNota', $infoNotaFiscal));
        } catch (\Exception $e) {
            //var_dump($e);
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error('Ocorreram erros ao realizar o fechamento da nota fiscal.', $e->getCode()));
            $erros = $this->lancamentoNotafiscalService->getErrosAvisos();
            if(empty($erros)) {
               $erros = array(array("CDERROAVISO" => "", "DSERROAVISO" => $e->getMessage(), "IDERROAVISO" => "B")); 
            }
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosFechamento', $erros));
        }
    }
    
    public function geraRateio(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $campos = array();
            foreach ($conditions as $campo){
                $campos[$campo['columnName']] = $campo['value'];
            }
            $infoRateio = $this->lancamentoNotafiscalService->geraRateio($campos['FILIAL'], $campos['LANCAMENTO'], $campos['IDENTRSAID'], $campos['RATEIONF'], $campos['GERARATAUT'], $campos['REALIZAMANUT']);
            //var_dump($infoRateio);die;
            //$response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('geraRateio', $infoRateio));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        //$filial, $lancamento, $idEntrSaid,  $rateioNF, $autMan = 'N', $realizaManut = 'P'
    }

    public function calculaImpostoItemProduto(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $cabecalho = $rows[0];
            $item = $rows[1];
            
            $calculoImpostoProduto = $this->lancamentoNotafiscalService->calculaImpostoItemProduto($cabecalho, $item, false);

            $response->addDataset(new DataSet('calculoImpostoProduto', $calculoImpostoProduto));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaItensAf(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $afSelecionadas = $rows;

            $arrayProdutos = $this->lancamentoNotafiscalService->salvaItensAf($afSelecionadas);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('lancamento_nota_itens', $arrayProdutos));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function replicacaoManual(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $notafiscal = $rows[0];
            $documentos = $rows[1];
            $itens      = $rows[2];
            $replicacaoManual = $this->lancamentoNotafiscalService->replicacaoManual($notafiscal, $documentos, $itens);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('replicacaoManual', $replicacaoManual));
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvaInfoCompra(Request\Row $request, Response $response){
        $row  = $request->getRow();
        
        $this->lancamentoNotafiscalService->salvaInfoCompra($row);
    }
    
    public function editaInfoCompra(Request\Row $request, Response $response){
        $row  = $request->getRow();
        
        try{
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->lancamentoNotafiscalService->editaInfoCompra($row);
            
            $connection->commit();
            $this->entityManager->flush();
            
        }catch(\Exception $e){
           $connection->rollback(); 
        }
    }
    
    public function produtoLancamento(Request\Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        $cdSegMerc = '';
        
        foreach($conditions as &$condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
            }
            if($condition["columnName"] == 'CDARVPROD|NMPRODUTO'){
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace('.', '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace('.', '', $condition['value']);
                }
            }
        }
        
        $cdSegMerc = $this->lancamentoNotafiscalService->getSegMercFilial($cdFilial);
        if(isset($cdSegMerc[0])) {
            $cdSegMerc = $cdSegMerc[0]["CDSEGMER"];
        }
        
        if($cdSegMerc) {
            $filterSegmento = array("columnName" => "CDSEGMER", "operator" => "=", "value" => $cdSegMerc);
            array_push($conditions, $filterSegmento);
            $dataSourceName = 'prodfili_produto_segmercado';
        }
        else {
            $dataSourceName = 'prodfili_produto_semseg';
        }
        
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet('produto_filial', $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    private function preparaFiltroCNPJ(&$conditions, $rowsName){
        foreach($conditions as $key => &$condition){
            if($condition['columnName'] == $rowsName){
                $caracteresEspeciais = array("-", ".", "/");
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace($caracteresEspeciais, '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace($caracteresEspeciais, '', $condition['value']);
                }
            }
        }
        
        return $conditions;
    }
    
    public function getFornecedores(Request\Filter $request, Response $response) {
        $franqueadora = $this->lancamentoNotafiscalService->getFranqueadora();
        
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $conditions = $this->preparaFiltroCNPJ($conditions, "NRINSJURFORN|NMRAZSOCFORN|NMFANTFORN");
        
        $filterCriteria = new FilterCriteria('', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
        if($franqueadora) {
            $this->dataSourceName = 'fornecedor_lancamento_notafiscal_filial';
            $request->getFilterCriteria()->addCondition("P_CDFILIAL", "=", $this->environment->getCdfilial());
            parent::find($request, $response);
        }
        else {
            $this->dataSourceName = 'fornecedor_lancamento_notafiscal';
            parent::find($request, $response);
        }
    }
    
    public function getClientes(Request\Filter $request, Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $conditions = $this->preparaFiltroCNPJ($conditions, 'NRINSJURCLIE|NMRAZSOCCLIE|NMFANTCLIE');
        $this->dataSourceName = 'cliente_lancamento_notafiscal';
        
        $filterCriteria = new FilterCriteria('cliente_lancamento_notafiscal', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        
        parent::find($request, $response);
    }

    public function getCentcustRateio(Request\Filter $request, Response $response) {
        $this->dataSourceName = 'centcust_rateio';
    
        parent::find($request, $response);
        
        $rows = $response->getDataSets()[0]->getRows();
        
        if(count($rows) == 0) {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            
            foreach($conditions as &$condition) {
                if($condition["columnName"] == "CDFILIAL") {
                    $condition["value"] = "T";
                }
            }
            $filterCriteria = new FilterCriteria('centcust_rateio', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
            $request = new Request\Filter($filterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::find($request, $response);
        }
    }

    public function getDefaultLancamentoNota(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $cdFilial = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : ''; 
        $idEntrSaidOp = isset($row["CDFILIAL"]) ? $row["CDFILIAL"] : 'E';
        
        $defaultFilter = $this->lancamentoNotafiscalService->getDefaultNovaNota($row["CDFILIAL"], $row["IDENTRSAIDOP"]);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultNovaNota', $defaultFilter));
    }
    
    public function copiarNF(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            if (isset($row["IDORIGDESTOP"])){
                $tipoDestino = $row["IDORIGDESTOP"];
            }else if(isset($row["IDORIGDESTNF"])) {
                 $tipoDestino = $row["IDORIGDESTNF"];
            }
            
            switch ($tipoDestino) {
            case "C":
                $cliforn = $row["CDCLIENTE"];
                break;
            case "F":
                 $cliforn = $row["CDFORNECED"];
                break;
            case "L":
                 $cliforn = $row["CDFILIRECE"];
                break;
            }
            
            $paramfilial = $this->getParamFilial($row["CDFILIAL"]);
            $atualizaestoque = $this->getAtualizaEstoque($paramfilial, $row["CDFILIAL"], $row["CDTIPOOPER"]);
            $row['DTENTRSAID'] = $row['DTLANCAMENNF'];

            if(isset($row["IDTPIJURTRAN"])){
                $proxLancamento = $this->lancamentoNotafiscalService->geraCorpoNota(
                    $row["CDFILIAL"],  $row["CDFILIEMIT"],$row["CDEMPRESA"] , $row["CDTIPOOPER"], $row["CDSERIE"], $cliforn, $row["IDSISTEMA"], $atualizaestoque, 
                    '', $row["IDTPIJURTRAN"], $row["NRINSJURTRAN"], $row["NRPLACAVEIC"], $row["SGUFVEIC"], $row["IDPGTOFRETE"], $row["DSQTDEVOLU"], $row["DSESPECVOLU"], $row["DSMARCAVOLU"], $row["DSNUMEVOLU"], '',
                    $row["DTENTRSAID"], $row["VRPBRUTOVOLU"], $row["VRPLIQUVOLU"], '', $row["DTLANCAMENNF"]);
            }else{
                $proxLancamento = $this->lancamentoNotafiscalService->geraCorpoNota(
                    $row["CDFILIAL"],  $row["CDFILIEMIT"],$row["CDEMPRESA"] , $row["CDTIPOOPER"], $row["CDSERIE"], $cliforn, $row["IDSISTEMA"], $atualizaestoque, 
                    '', '',  null ,null , null, null, null, null, null, $row["DSNUMEVOLU"], '',
                    $row["DTENTRSAID"], $row["VRPBRUTOVOLU"], $row["VRPLIQUVOLU"], '', $row["DTLANCAMENNF"]);
            } 
            $prxLanctoNF = $proxLancamento["P_PRXLANCTONF"];
            $prxLancEstq = $proxLancamento["P_PRXLANCESTQ"];
            
            $itens = $this->lancamentoNotafiscalService->getItensNFOrigem($row["CDFILIAL"],$row["NRLANCTONF"]);
            foreach($itens as $item){
                $lote = ''; 
                $subLote= '';
                
                if ($row["LOTEORIG"]  === 'S'){
                    $lote    = $item["NRLOTEESTQ"];
                    $subLote = $item["NRSUBLOTE"];
                }
                
                $this->lancamentoNotafiscalService->procedureGeraNotaFiscalProduto(
                    $row["CDFILIAL"], $prxLanctoNF, $item["CDPRODUTO"], $item["CDPRODEMIS"], $item["DSPRODEMIS"],  
                    $item["QTITEMPROD"], $item["VRUNITITEM"],  $item["VRDESCITEM"], '', '', $lote, $subLote, null,
                    $item["IDORIGEMPROD"], $item["IDATUESTPROD"],  0, $item["CDMARCA"], null);
                    
                    
                $idutildtproc = $this->getParamGeral()->getIdutildtproc();
                $dtprocessamento = $paramfilial->getDtprocessa();
                
                if($idutildtproc ==='S' && (isset($dtprocessamento)  && $dtprocessamento != null)) {
                    $dtprocessamento =  $dtprocessamento->format('d/m/Y');   
                }else {
                    $dtprocessamento = date('d/m/y');
                }
                
                $this->lancamentoNotafiscalService->procedureGeraFechamentoSaida( $row["CDFILIAL"], $prxLanctoNF,  $dtprocessamento, 
                                                                                    $prxLancEstq, '', 'S', 'N');
            }
            
            $impostObg =  $this->verifImpObrig( $row["CDFILIEMIT"], $row["CDFILIAL"], $prxLanctoNF);
            if(!empty($impostObg)){
              $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosCopiarNF', $this->erroImpoObg($impostObg)));
              throw new \Exception("Erro ao copiar Nota Fiscal.");
            }else{
                $connection->commit();
                $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('copiarNF', array('success' => 'Nota Fiscal copiada com sucesso')));  
            }

        }catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $this->entityManager->rollback();
        }
    }
    
    public function getAtualizaEstoque($paramfilial, $cdfilial, $tipoOperacao){
        $atualizaestoque = 'S';

        if($paramfilial->getIdctrlestq() == 'N'){
            $atualizaestoque = 'N';
        } else {
            $operacao = $this->getTipooperac($tipoOperacao);  
            if($operacao->getIdatuesttpop() === 'N'){
                $atualizaestoque = 'N';
            }
        }
        
        return $atualizaestoque;
    }
    
    public function utilizaControleFiscal($paramfilial, $paramGeral) {
        if ($paramGeral->getIdutctrlfissaid() === 'S')
            return true;
        else if ($paramfilial->getIdutctrlfissaid() === 'S')
            return true;
        else 
            return false;
    }
    
    public function verifImpObrig( $cdfiliemit, $cdfilial, $prxLanctoNF) {
        $ProdutosSemImpObrig = array();
        if ($this->utilizaControleFiscal( $this->getParamFilial($cdfiliemit) , $this->getParamGeral())){
            $nf = $this->lancamentoNotafiscalService->getDadosNF($cdfilial, $prxLanctoNF);
            $impoObrig =  $this->lancamentoNotafiscalService->impostoObrig($nf[0]["IDENTRSAIDOP"], $nf[0]["CDTIPOOPER"]);
            foreach ($impoObrig as $imposto){
                if ($imposto["IDPRODNOTA"] == 'P'){
                    $impostoProduto = $this->lancamentoNotafiscalService->impostoProduto($cdfilial, $prxLanctoNF,$imposto["CDIMPOSTO"]);
                    if(isset($impostoProduto[0])){
                        $impostoProduto = $impostoProduto[0];
                        if(isset($ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]])){
                           $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["SGIMPOSTO"] .= ',' . $imposto["SGIMPOSTO"];
                        }else{
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["ERROIMPOPROD"] = 'O produto não possui os impostos obrigatórios: '. $imposto["SGIMPOSTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["NMPRODUTO"] = $impostoProduto["NMPRODUTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["CDPRODUTO"] = $impostoProduto["CDPRODUTO"];
                            $ProdutosSemImpObrig[$impostoProduto["CDARVPROD"]]["CDARVPROD"] = $impostoProduto["CDARVPROD"];
                        }
                }
                }
            }
        }
        return $ProdutosSemImpObrig; 
    }
    
    public function getParamFilial($cdfilial){
       return  $this->entityManager->getRepository('Geral\Entities\Paramfilial')->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab(), 'cdfilial' => $cdfilial));
    }
    
    public function getTipooperac($tipoOperacao){
        return $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Tipooperac')->findOneBy(
            array('identrsaidop' => 'S',
                  'cdtipooper' => $tipoOperacao));
    }
    
    public function getParamGeral(){
        return  $this->entityManager->getRepository('Teknisa\Model\Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $this->environment->getNrOrgTrab()));
    }
    
    public function erroImpoObg($impostObg){
            $produtosSemImposto = array();
            foreach($impostObg as $imposto){
               array_push($produtosSemImposto,$imposto);
            }
            return $produtosSemImposto;
    }
    
    public function produtoImportarLancamento(Request\Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $cdFilial = '';
        $cdSegMerc = '';
        
        foreach($conditions as &$condition) {
            if($condition["columnName"] == "CDFILIAL") {
                $cdFilial = $condition["value"];
            }
            if($condition["columnName"] == 'CDARVPROD|NMPRODUTO'){
                if($condition['operator'] == "MAPPED_LIKE_ALL"){
                    $condition['value'][0]['_ALL'] = str_replace('.', '', $condition['value'][0]['_ALL']);
                } else{
                    $condition['value'] = str_replace('.', '', $condition['value']);
                }
            }
        }
        
        $cdSegMerc = $this->lancamentoNotafiscalService->getSegMercFilial($cdFilial);
        
        if(isset($cdSegMerc[0])) {
            $cdSegMerc = $cdSegMerc[0]["CDSEGMER"];
        }
        
        if($cdSegMerc) {
            $dataSourceName = 'prodfili_produto_prodfornxml_produtoseg';
        }
        else {
            $dataSourceName = 'prodfili_produto_prodfornxml';
        }
        
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());

        $dataSet = new DataSet($dataSourceName, $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    public function deleteImpostoItem(Request\Row $request, Response $response) {
            try {
            $row = $request->getRow();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->lancamentoNotafiscalService->deleteImpostoNF($row["CDFILIAL"], $row["NRLANCTONF"], $row["CDIMPOSTO"]);
            $this->lancamentoNotafiscalService->deleteImpostoItem($row["CDFILIAL"], $row["NRLANCTONF"], $this->environment->getNrOrgTrab(),  $row["NRITPROIMPOS"],  $row["NRSEQITPROD"], $row["CDIMPOSTO"]);
            
            $this->entityManager->flush();
            $connection->commit();
            return true;
        }
        catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        }
        
    }    
    
    public function salvarImpostoItem(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow()[0];
            $this->lancamentoNotafiscalService->insereImpostoItem($row);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function insereItemprodutoNFDevolucao(Request\Row $request, Response $response) {
        try {
            
            $row = $request->getRow();
            $this->lancamentoNotafiscalService->insereItemprodutoNFDevolucao($row["CDFILIAL"], $row["NRLANCTONF"], $row["NRORG"]);
            
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function findFilial(Request\Filter $request, Response $response) {
        $this->switchDataSource('filial');

        parent::find($request, $response);
        $this->switchDataSource('default');
    }
    
    public function find_prod_fili(Request\Filter $request, Response $response) {
        $this->switchDataSource('prod_fili');
        parent::find($request, $response);
        
    }

    private function switchDataSource($label) {
        $this->dataSourceName = $this->dataSourceNames[$label];
    }
    
    public function getDataForUnidade(Request\Filter $request, Response $response) {
        #default values
        $data = null;

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();
        
        #tries to get branch parametrization data
        $data = $this->lancamentoNotafiscalService->getDataForUnidade($CDFILIAL, $NRORG);

        #returns data
        $response->addDataSet(new DataSet("paramestoque_unidade", $data));
        
    }
    
    public function getDataForGeral(Request\Filter $request, Response $response) {
        #default values
        $data = null;

        #get values
        $NRORG = $this->environment->getNrOrgTrab();
       
        #tries to get generic parametrization data
        $data = $this->lancamentoNotafiscalService->getDataForGeral($NRORG);
        
        #returns data
        $response->addDataSet(new DataSet("paramestoque_geral", $data));
    }

    private function buscaItensCombs($filial, $lancamento) {
        $conditions = [["columnName" => "P_CDFILIAL"   , "operator" => "=", "value" => $filial],
                       ["columnName" => "P_NRLANCTONF" , "operator" => "=", "value" => $lancamento]];
        $dataSourceName = 'nota_combs';
        
        $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        return $result->getRows();
    }
    
    public function salvaMultiItensLancamentoSaida(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $novosItens = $rows[0];
            $itensAlterados = $rows[1];
            $cabecalho = $rows[2];
            $impostos = $this->lancamentoNotafiscalService->salvaMultiItensLancamentoSaida($novosItens, $itensAlterados, $cabecalho);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getInfoTipoOper(Request\Row $request, Response $response){
            $row = $request->getRow();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $data = $this->lancamentoNotafiscalService->getInfoTipoOper($row["CDTIPOOPER"]);
            $connection->commit();
            
            $response->addDataSet(new DataSet("getInfoTipoOper", $data));
    }
    
    // Busca o status da nota
    public function getStatusNota(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $status = $this->lancamentoNotafiscalService->getStatusNota($row['CDFILIAL'], $row['NRLANCTONF'], $row['NRORG']);
        
        $response->addDataSet(new DataSet("getStatusNota",  $status));
    }
    
    /*---------------------------------- Inicio - Importação de Requisição por Setor ----------------------------------*/
    public function getRequisicoesPorSetor(Request\Row $request, Response $response){
        $CDFILIAL = $request->getRow()["CDFILIAL"];
        $CDSETOR = $request->getRow()["CDSETOR"];
        $requisicoes = $this->lancamentoNotafiscalService->getRequisicoesPorSetor($CDFILIAL, $CDSETOR);
        $response->addDataSet(new DataSet("RequisicoesSetor",  $requisicoes));
    }
    
    public function getItensPorRequisicaoDeSetor(Request\Row $request, Response $response){
        $CDFILIAL = $request->getRow()["CDFILIAL"];
        $NRREQUESTOS = $request->getRow()["NRREQUESTO"];
        forEach($NRREQUESTOS as $request){
            $NRREQUESTO = $request['NRREQUESTO'];
            $itens = $this->lancamentoNotafiscalService->getItensPorRequisicaoDeSetor($CDFILIAL, $NRREQUESTO);
            $this->itensRequisicao[] = $itens;
        }
        
        $response->addDataSet(new DataSet("RequisicoesSetorItens",  $this->itensRequisicao));
    }
    
    public function getSetorRequisicao(Request $request, Response $response){
        $setor = $this->lancamentoNotafiscalService->getSetorRequisicao();
            
        $response->addDataSet(new DataSet("SetorRequisicao",  $setor));
    }
    
    public function getVrItens_Requisicao(Request $request, Response $response){
        $row = $request->getRow();
        $vrItens = $this->lancamentoNotafiscalService->getVrItens_Requisicao($row);
        
        $response->addDataSet(new DataSet("vrItens",  $vrItens));
    }
    /*---------------------------------- Final - Importação de Requisição por Setor ----------------------------------*/
    public function getInscrmunicip(Request $request, Response $response){
        $row = $request->getRow();

        $localidade = $this->lancamentoNotafiscalService->getLocalidade($row["CDEMPRESA"], $row["CDINSCMUNI"]);

        foreach($localidade as $key => $field){
            $row[$key] = $field;
        }

        $response->addDataSet(new DataSet("localidade",  array($row)));
    }
}