<?php
namespace Teknisa\DF\Modules\Relatorios\Controllers;

use Teknisa\DF\Modules\Relatorios\Services\Dfe16000ConsistenciasAfNfService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Report\ReportService;

class Dfe16000ConsistenciasAfNfController extends \Zeedhi\Framework\Controller\Report {

    protected $consistenciasService;
    protected $dataSources = array (
        "DFE16000" => "notafiscal_consistencias_analitico"
    );

    public function __construct(ReportService $reportService, Dfe16000ConsistenciasAfNfService $consistenciasService) {
        $this->consistenciasService = $consistenciasService;
        parent::__construct($reportService);
    }
    

    protected $reportMapping = array(
        'DFE16000' => array(
            'strategy' => "BIRT",
            'parameterMapping' => array(
                'P_NRORG'=> "P_NRORG",
                'P_DTINICIAL'=> "P_DTINICIAL",
                'P_DTFINAL'=> "P_DTFINAL",
                'L_CDFILIRECE'=> "L_CDFILIRECE",
                'L_CHAVE'=> "L_CHAVE"
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->setNrorg($row);
            $this->consistenciasService->validateParameters($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getMainQueryData(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->setNrorg($row);
            $mainQueryData = $this->consistenciasService->getAnaliticoData($row);
            $response->addDataSet(new DataSet("mainQueryData", $mainQueryData));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function setNrorg($row) {
        $row["P_NRORG"] = $this->consistenciasService->getNrorg();
    }

}
