<?php
namespace Teknisa\DF\Services;

use Fiscal\Entities\Erroavisonf;
use Teknisa\Libs\Util\Environment;

class Ger03900ErrosAvisosNFService {

	protected $environment;
	protected $entityManager;

	public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager){
		$this->environment = $environment;
		$this->entityManager = $entityManager;
	}

	private function buildList() {
		$list = [
			"AV01",
			"AV02",
			"AV03",
			"AV04",
			"AV05",
			"AV06",
			"AV07",
			"AV08",
			"AV09",
			"AV10",
			"AV11",
			"AV12",
			"AV13",
			"AV14",
			"AV15",
			"AV16",
			"AV17",
			"ER01",
			"ER02",
			"ER03",
			"ER04",
			"ER05",
			"ER06"
		];
		return $list;
	}

	//Busca se existe algum registro na tabela 'ERROAVISONF'
	private function buscaLista ($nrorg) {
		$sucesso = false;

		try {
			$errosAvisosNF = Erroavisonf::findByNrorg($nrorg);
			if(count($errosAvisosNF) > 0) {
				$sucesso = true;
			}
		}
		catch (\Exception $e) {
			throw $e;
		}

		return $sucesso;
	}

	private function insereListaErrosAvisosNF ($nrorg, $operador, $data, $listaErrosAvisosNF){

		$this->entityManager->beginTransaction();

		try {

			foreach($listaErrosAvisosNF as $cderroaviso) {
				$erroAvisoNF = new Erroavisonf();

				$erroAvisoNF->setCderroaviso($cderroaviso);
				$erroAvisoNF->setIderroaviso('C');

				$erroAvisoNF->setNrorg($nrorg);
				$erroAvisoNF->setDtultatu($data);
				$erroAvisoNF->setCdoperultatu($operador);
				$erroAvisoNF->setNrorgultatu($nrorg);
				$erroAvisoNF->setDtinclusao($data);
				$erroAvisoNF->setCdoperinclusao($operador);
				$erroAvisoNF->setNrorginclusao($nrorg);

				$erroAvisoNF->persist();
			}
			$this->entityManager->flush();
			$this->entityManager->commit();

		} catch (\Exception $e) {
			$this->entityManager->rollback();
		}
	}

	public function validateDatabase() {
		$nrorg = $this->environment->getNrOrgTrab();
		if (!$this->buscaLista($nrorg)) {
			$operador = $this->environment->getCdOperador();
			$currentTime = $this->environment->getCurrentTime();
			$listaErrosAvisosNF = $this->buildList();
			$this->insereListaErrosAvisosNF($nrorg, $operador, $currentTime, $listaErrosAvisosNF);
		}
	}

}